% Calcualte rangeDoppler from radar amplitude and phase baseband
% frames. rangeDoppler is calculated using the STFT (Short Time Fourier
% Transform) to extract time and frequency data (2 dimentions) from each
% frame (1 dimention)
%
% @param[in] hdrMat
% @param[in] frames
% @param[in] radarSettings
%
% @param[out] rangeDoppler      - [M,numRangeBins,numFrames] matrix
% @param[out] r                 - [1,numRangeBins] frame range axis
% @param[out] f                 - [1,M] doppler frequency axis

function [rangeDoppler, r, f] = getPulseDoppler(hdrMat,frames,radarSettings)
    timer = tic;

    binLen = hdrMat(1,3);
    rangeOffset = hdrMat(1,6);
    winLen = radarSettings.winLen;   % STFT window lenght [s]
    stftHop = radarSettings.stftHop; % frames between each rangeDoppler matrix computation (can minimum be 1)
    prf = radarSettings.prf;
    
    % input sanitation
    if isempty(frames)
        error('input <frames> empty, could not compute rangeDoppler');
    end
    [numFrames,numRangeBins] = size(frames);

    % determine window size
    if winLen > 50
        windowSize = winLen;
        Output.INFO(['window length is ',num2str(windowSize/prf,'%.3f'),'s, containing ',num2str(windowSize),' frames','\n']);

    elseif winLen < 50 && winLen > 0
        windowSize = getNumFramesInWindow(winLen,prf);
        Output.INFO(['window length is ',num2str(winLen,'%.3f'),'s, containing ',num2str(windowSize),' frames','\n']);

    else
        error(num2ste(winLen),' is not a valid winLen input');
    end

    % frameBuffer/ window indexes
    winIndexBottom = floor(windowSize/2); % indx of first element in bottom half of window
    winIndexTop = windowSize-winIndexBottom-1; % index of last element in top half of window

    % axes
    r = linspace(rangeOffset,binLen*numRangeBins,numRangeBins);
    f = linspace(-prf/2,prf/2,windowSize);


    numRangeDopplerMat = length(1:stftHop:numFrames);
    rangeDoppler = zeros(windowSize,numRangeBins,numRangeDopplerMat);
    gwindow = gausswin(windowSize)';

    % calculate first rangeDoppler with a full window (start at frame M/2+1)
    startFrame = 1; %ceil(M/2);

    rangeDopplerIndex = 1;
    for h = startFrame:stftHop:numFrames
        % zero-pad the first and last frames to get full window length
        if h <= winIndexBottom
            frameBuffer = ([ zeros((winIndexBottom+1-h),numRangeBins); frames(1:(h+winIndexTop),:)]');
        elseif (h+winIndexTop) >= numFrames
            frameBuffer = ([ frames((h-winIndexBottom):numFrames,:); zeros((winIndexTop+h)-numFrames,numRangeBins)]');
        else
            frameBuffer = frames((h-winIndexBottom):(h+winIndexTop),:)';
        end

        for n = 1:numRangeBins
            % Stationary Clutter
            clutter = sum(frameBuffer(n,:))/windowSize; % Frame average at range bin n. Subtract from framebuffer to remove 0Hz clutter.

            y1 = (frameBuffer(n,:)-clutter);

            % window frame buffer
            y2 = y1 .* gwindow;

            % Time Frequency Analysis (STFT)
            y3(:,n) = fftshift(fft(y2,windowSize)/windowSize); % normalise fft amplitudes
        end % numRangeBins
        rangeDoppler(:,:,rangeDopplerIndex) = abs(y3.^2);% temp'; % transpose to get range as x vector

        rangeDopplerIndex = rangeDopplerIndex+1;
    end % numFrames

    S = whos('rangeDoppler');
    Output.VERBOSE([num2str(rangeDopplerIndex),' rangeDopplers computed in ',num2str(toc(timer),'%.1f'),'s, hop=',num2str(stftHop),', size=',int2str(S.bytes/1e6),'MB''\n']);
end